@@ -2,7 +2,7 @@
  **
  ** Copyright (C) 1991-2007 Heinz van Saanen
  **
- ** This file is part of the the function viewer Zhu3D.
+ ** This file is part of the function viewer Zhu3D.
  **
  ** This file may be used under the terms of the GNU General Public
  ** License version 2 as published by the Free Software Foundation
@@ -51,21 +51,29 @@ using namespace std;
 using namespace Eigen;
 using namespace OpenBabel;
 
+#if defined(Q_CC_MSVC)
+#define AVOGADRO_ALIGNMENT16
+#else
+#define AVOGADRO_ALIGNMENT16 __attribute__((aligned(16)))
+#endif
+
 namespace Avogadro
 {
-
+#ifdef Q_CC_MSVC
+#pragma pack(push, 16)
+#endif
   // ****************************************************************************
   // LOCAL CONSTANT/TABLE-STUFF
   // ****************************************************************************
 
   // Z-Translation, original value was 48.0
-  const float IsoGen::fTargetValue __attribute__((aligned(16))) = 0.0f;
+  const float IsoGen::fTargetValue AVOGADRO_ALIGNMENT16 = 0.0f;
 
   // These tables are used so that everything can be done in little loops that
   // you can look at all at once rather than in pages and pages of unrolled code
   // a2fVertexOffset lists the positions, relative to vertex0, of each of the
   // eight vertices of a cube
-  const float IsoGen::a2fVertexOffset[8][3] __attribute__ ((aligned(16))) =
+  const float IsoGen::a2fVertexOffset[8][3] AVOGADRO_ALIGNMENT16 =
   {
     {0.0f, 0.0f, 0.0f},
     {1.0f, 0.0f, 0.0f},
@@ -77,7 +85,7 @@ namespace Avogadro
     {0.0f, 1.0f, 1.0f}
   };
 
-  const int IsoGen::a2iVertexOffset[8][3] __attribute__ ((aligned(16))) =
+  const int IsoGen::a2iVertexOffset[8][3] AVOGADRO_ALIGNMENT16 =
   {
     {0, 0, 0},
     {1, 0, 0},
@@ -91,7 +99,7 @@ namespace Avogadro
 
   // a2iEdgeConnection lists the index of the endpoint vertices for each of the
   // twelve edges of the cube
-  const long IsoGen::a2iEdgeConnection[12][2] __attribute__((aligned(16))) =
+  const long IsoGen::a2iEdgeConnection[12][2] AVOGADRO_ALIGNMENT16 =
   {
     {0,1},
     {1,2},
@@ -109,7 +117,7 @@ namespace Avogadro
 
   // a2fEdgeDirection lists the direction vector (vertex1-vertex0) for each edge
   // in the cube
-  const float IsoGen::a2fEdgeDirection[12][3] __attribute__((aligned(16))) =
+  const float IsoGen::a2fEdgeDirection[12][3] AVOGADRO_ALIGNMENT16 =
   {
     { 1.0f, 0.0f, 0.0f},
     { 0.0f, 1.0f, 0.0f},
@@ -128,7 +136,7 @@ namespace Avogadro
   // a2iTetrahedronEdgeConnection lists the index of the endpoint vertices for
   // each of the six edges of the tetrahedron
   const long IsoGen::a2iTetrahedronEdgeConnection[6][2]
-    __attribute__((aligned(16))) =
+   AVOGADRO_ALIGNMENT16 =
   {
     {0,1},
     {1,2},
@@ -140,7 +148,7 @@ namespace Avogadro
 
   // a2iTetrahedronEdgeConnection lists the index of vertices from a cube that
   // made up each of the six tetrahedrons within the cube
-  const long IsoGen::a2iTetrahedronsInACube[6][4] __attribute__((aligned(16))) =
+  const long IsoGen::a2iTetrahedronsInACube[6][4] AVOGADRO_ALIGNMENT16 =
   {
     {0,5,1,6},
     {0,1,2,6},
@@ -158,7 +166,7 @@ namespace Avogadro
   // This table lists the edges intersected by the surface for all 16 possible vertex states
   // There are 6 edges.  For each entry in the table, if edge #n is intersected,
       // then bit #n is set to 1
-  const long IsoGen::aiTetrahedronEdgeFlags[16] __attribute__((aligned(16))) =
+  const long IsoGen::aiTetrahedronEdgeFlags[16] AVOGADRO_ALIGNMENT16 =
   {
     0x00, 0x0d, 0x13, 0x1e, 0x26, 0x2b, 0x35, 0x38, 0x38, 0x35, 0x2b, 0x26, 0x1e, 0x13, 0x0d, 0x00,
   };
@@ -167,7 +175,7 @@ namespace Avogadro
   // specific triangulation of the edge intersection points.
   // a2iTetrahedronTriangles lists all of them in the form of
   // 0-2 edge triples with the list terminated by the invalid value -1
-  const long IsoGen::a2iTetrahedronTriangles[16][7] __attribute__((aligned(16))) =
+  const long IsoGen::a2iTetrahedronTriangles[16][7]AVOGADRO_ALIGNMENT16 =
   {
     {-1, -1, -1, -1, -1, -1, -1},
     {0, 3, 2, -1, -1, -1, -1},
@@ -196,7 +204,7 @@ namespace Avogadro
   // For any cube the are 2^8=256 possible sets of vertex states
   // This table lists the edges intersected by the surface for all 256 possible vertex states
   // There are 12 edges.  For each entry in the table, if edge #n is intersected, then bit #n is set to 1
-  const long IsoGen::aiCubeEdgeFlags[256] __attribute__((aligned(16))) =
+  const long IsoGen::aiCubeEdgeFlags[256] AVOGADRO_ALIGNMENT16 =
   {
     0x000, 0x109, 0x203, 0x30a, 0x406, 0x50f, 0x605, 0x70c, 0x80c, 0x905, 0xa0f, 0xb06, 0xc0a, 0xd03, 0xe09, 0xf00,
     0x190, 0x099, 0x393, 0x29a, 0x596, 0x49f, 0x795, 0x69c, 0x99c, 0x895, 0xb9f, 0xa96, 0xd9a, 0xc93, 0xf99, 0xe90,
@@ -223,7 +231,7 @@ namespace Avogadro
   // For example: a2iTriangleConnectionTable[3] list the 2 triangles formed when corner[0]
   // and corner[1] are inside of the surface, but the rest of the cube is not
   const long IsoGen::a2iTriangleConnectionTable[256][16]
-    __attribute__((aligned(16))) =
+    AVOGADRO_ALIGNMENT16 =
   {
     {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
     {0, 8, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
@@ -482,7 +490,9 @@ namespace Avogadro
     {0, 3, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1},
     {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}
   };
-
+#ifdef Q_CC_MSVC
+#pragma pack(pop)
+#endif
   // ****************************************************************************
   // THREAD RELATED STUFF
   // ****************************************************************************
@@ -567,22 +577,22 @@ namespace Avogadro
       switch(pd->painter()->quality())
       {
       case 0:
-        m_stepSize = 1.0;
+        m_stepSize = 1.0f;
         break;
       case 1:
-        m_stepSize = 0.5;
+        m_stepSize = 0.5f;
         break;
       case 2:
-        m_stepSize = 0.3;
+        m_stepSize = 0.3f;
         break;
       case 3:
-        m_stepSize = 0.22;
+        m_stepSize = 0.22f;
         break;
       case 4:
-        m_stepSize = 0.15;
+        m_stepSize = 0.15f;
         break;
       default:
-        m_stepSize = 0.10;
+        m_stepSize = 0.10f;
       }
     }
     m_min = Vector3f(m_grid->grid()->GetOriginVector().x(),
@@ -623,10 +633,16 @@ namespace Avogadro
   void IsoGen::vMarchCube1(const float fX, const float fY, const float fZ)
   {
     long iTriangle, iEdge, iEdgeFlags, iFlagIndex=0;
-    Vector3f asEdgeVertex[12] __attribute__((aligned(16)));
-    Vector3f asEdgeNorm[12] __attribute__((aligned(16)));
-    float fOffset __attribute__((aligned(16)));
-    float afCubeValue[8] __attribute__((aligned(16)));
+#ifdef Q_CC_MSVC
+#pragma pack(push, 16)
+#endif
+    Vector3f asEdgeVertex[12] AVOGADRO_ALIGNMENT16;
+    Vector3f asEdgeNorm[12] AVOGADRO_ALIGNMENT16 ;
+    float fOffset AVOGADRO_ALIGNMENT16 ;
+    float afCubeValue[8] AVOGADRO_ALIGNMENT16 ;
+#ifdef Q_CC_MSVC
+#pragma pack(pop)
+#endif
 
     // Check we have a valid grid
     if (m_grid->grid() == 0)
@@ -736,11 +752,16 @@ namespace Avogadro
   void IsoGen::vMarchCube1(int i, int j, int k)
   {
     long iTriangle, iEdge, iEdgeFlags, iFlagIndex=0;
-    Vector3f asEdgeVertex[12] __attribute__((aligned(16)));
-    Vector3f asEdgeNorm[12] __attribute__((aligned(16)));
-    float fOffset __attribute__((aligned(16)));
-    float afCubeValue[8] __attribute__((aligned(16)));
-
+#ifdef Q_CC_MSVC
+#pragma pack(push, 16)
+#endif
+    Vector3f asEdgeVertex[12] AVOGADRO_ALIGNMENT16 ;
+    Vector3f asEdgeNorm[12] AVOGADRO_ALIGNMENT16 ;
+    float fOffset AVOGADRO_ALIGNMENT16 ;
+    float afCubeValue[8] AVOGADRO_ALIGNMENT16 ;
+#ifdef Q_CC_MSVC
+#pragma pack(pop)
+#endif
     // Work out the cube edge location
     float fX = i * m_stepSize + m_min.x();
     float fY = j * m_stepSize + m_min.y();
@@ -871,10 +892,16 @@ namespace Avogadro
 void IsoGen::vMarchCube2(const float fX, const float fY, const float fZ)
   {
     long iVertex, iTetrahedron, iVertexInACube;
-    Vector3f asCubePosition[8] __attribute__((aligned(16)));
-    Vector3f asTetrahedronPosition[4] __attribute__((aligned(16)));
-    float afCubeValue[8] __attribute__((aligned(16)));
-    float afTetrahedronValue[4] __attribute__((aligned(16)));
+#ifdef Q_CC_MSVC
+#pragma pack(push, 16)
+#endif
+    Vector3f asCubePosition[8] AVOGADRO_ALIGNMENT16 ;
+    Vector3f asTetrahedronPosition[4] AVOGADRO_ALIGNMENT16 ;
+    float afCubeValue[8] AVOGADRO_ALIGNMENT16 ;
+    float afTetrahedronValue[4] AVOGADRO_ALIGNMENT16 ;
+#ifdef Q_CC_MSVC
+#pragma pack(pop)
+#endif
 
     // Make a local copy of the cube's corner positions
     for(iVertex=0; iVertex<8; iVertex++)
@@ -908,10 +935,16 @@ void IsoGen::vMarchCube2(const float fX, const float fY, const float fZ)
   void IsoGen::vMarchTetrahedron(GLvector *pasTetrahedronPosition, const float *pafTetrahedronValue)
   {
     long iEdge, iVert0, iVert1, iEdgeFlags, iFlagIndex=0;
-    float fOffset __attribute__((aligned(16)));
-    float fInvOffset __attribute__((aligned(16)));
-    Vector3f asEdgeVertex[6] __attribute__((aligned(16)));
-    Vector3f asEdgeNorm[6] __attribute__((aligned(16)));
+#ifdef Q_CC_MSVC
+#pragma pack(push, 16)
+#endif
+    float fOffset AVOGADRO_ALIGNMENT16 ;
+    float fInvOffset AVOGADRO_ALIGNMENT16 ;
+    Vector3f asEdgeVertex[6] AVOGADRO_ALIGNMENT16 ;
+    Vector3f asEdgeNorm[6] AVOGADRO_ALIGNMENT16 ;
+#ifdef Q_CC_MSVC
+#pragma pack(pop)
+#endif
 
     // Find which vertices are inside of the surface and which are outside
     if(pafTetrahedronValue[0] <= fTargetValue) iFlagIndex |= 1;
