@@ -1,9 +1,13 @@
 /***************************************************************************
-                          fractionbasewidget.cpp  -  base fraction painting
+                 fractionbasewidget.cpp  -  base fraction painting
                              -------------------
     begin                : 2004/05/30
     copyright            : (C) 2004 by Sebastian Stein
     email                : seb.kde@hpfsc.de
+
+    copyright            : (C) 2008 by Tiago Porangaba, tiago.porangaba@ltia.fc.unesp.br
+    copyright            : (C) 2008 by Tadeu Araujo, tadeu.araujo@ltia.fc.unesp.br
+    copyright            : (C) 2008 by Danilo Balzaque, danilo.balzaque@ltia.fc.unesp.br
  ***************************************************************************/
 
 /***************************************************************************
@@ -26,11 +30,11 @@
 
 #include "settingsclass.h"
 
-FractionBaseWidget::FractionBaseWidget(QWidget * parent = 0, const char * name = 0) :
-			QWidget(parent, name)
+FractionBaseWidget::FractionBaseWidget(QWidget * parent = 0) :
+			QWidget(parent)
 {
 #ifdef DEBUG
-	kdDebug() << "constructor FractionBaseWidget" << endl;
+	kDebug() << "constructor FractionBaseWidget";
 #endif
 
 	// set colors and font used for task displaying
@@ -40,7 +44,7 @@ FractionBaseWidget::FractionBaseWidget(QWidget * parent = 0, const char * name =
 FractionBaseWidget::~FractionBaseWidget()
 {
 #ifdef DEBUG
-	kdDebug() << "destructor FractionBaseWidget" << endl;
+	kDebug() << "destructor FractionBaseWidget";
 #endif
 }
 
@@ -50,7 +54,7 @@ void FractionBaseWidget::updateAndRepaint()
 	update();
 }
 
-void FractionBaseWidget::paintRatio(QPainter & paint, ratio tmp_ratio, int & x_pos, QFontMetrics & fm, bool show_mixed, bool addMargin)
+void FractionBaseWidget::paintRatio(QPainter & paint, ratio tmp_ratio, int & x_pos, int & y_pos, QFontMetrics & fm, bool show_mixed, bool addMargin, bool show_center)
 {
 	QPen pen = paint.pen(); // get the pen
 	int fontHeight = fm.lineSpacing(); // get the font height
@@ -68,14 +72,14 @@ void FractionBaseWidget::paintRatio(QPainter & paint, ratio tmp_ratio, int & x_p
 	//         5             5
 	int_numerator = tmp_ratio.numerator();
 	int_denominator = tmp_ratio.denominator();
-	if (show_mixed == true && QABS(int_numerator) >= QABS(int_denominator))
+	if (show_mixed == true && qAbs(int_numerator) >= qAbs(int_denominator))
 	{
 		// calculate the mixed number
 		int_mixed = int(int_numerator / int_denominator);
 
 		// the negative sign is in front of the mixed number
-		int_numerator = QABS(int_numerator);
-		int_denominator = QABS(int_denominator);
+		int_numerator = qAbs(int_numerator);
+		int_denominator = qAbs(int_denominator);
 
 		// we have to reduce the numerator by the mixed number * denominator
 		int_numerator = int_numerator % int_denominator;
@@ -86,13 +90,13 @@ void FractionBaseWidget::paintRatio(QPainter & paint, ratio tmp_ratio, int & x_p
 		str_denominator.setNum(int_denominator);
 
 		// paint the front part of the mixed number
-		paintMiddle(paint, str_mixed, x_pos, fm, m_colorNumber);
+		paintMiddle(paint, str_mixed, x_pos, y_pos, fm, m_colorNumber);
 	} else {
 
 		// don't show the ratio as mixed number
 		str_numerator.setNum(int_numerator);
 		str_denominator.setNum(int_denominator);
-	} // if (show_mixed == true && QABS(int_numerator) > QABS(int_denominator))
+    	} // if (show_mixed == true && qAbs(int_numerator) > qAbs(int_denominator))
 
 	// get the text width of the current ratio
 	fontWidth = fm.width(str_numerator);
@@ -107,16 +111,25 @@ void FractionBaseWidget::paintRatio(QPainter & paint, ratio tmp_ratio, int & x_p
 	// make sure we don't display something like:   0
 	//                                            7 -
 	//                                              3
-	if (! (int_numerator == 0 && show_mixed == true) )
+	if (! (int_numerator == 0) )
 	{
-		// paint the numerator
-		paint.drawText(x_pos, 0, fontWidth, fontHeight, AlignCenter, str_numerator);
-	
-		// paint the fraction line between numerator and denominator
-		paint.fillRect(x_pos, fontHeight + 4, fontWidth, 2, m_colorLine);
-	
-		// paint the denominator
-		paint.drawText(x_pos, fontHeight + 10, fontWidth, fontHeight, AlignCenter, str_denominator);
+		if ( show_center == true )
+			x_pos = 80 - fontWidth/2;
+		if (int_denominator != 1)
+		{
+			// paint the numerator
+			paint.drawText(x_pos, y_pos, fontWidth, fontHeight, Qt::AlignCenter, str_numerator);
+			// paint the fraction line between numerator and denominator
+			paint.fillRect(x_pos, y_pos + fontHeight + 4, fontWidth, 2, m_colorLine);
+			// paint the denominator
+			paint.drawText(x_pos, y_pos + fontHeight + 10, fontWidth, fontHeight, Qt::AlignCenter, str_denominator);
+		}
+		else
+		{
+			// paint the numerator and move the y position down to align with the + signal
+			paint.drawText(x_pos, y_pos + fontHeight - fontHeight / 2, fontWidth,
+				fontHeight, Qt::AlignCenter, str_numerator);
+		}
 	
 		// move the x position to the right by adding the width used for painting
 		// the ratio and a margin
@@ -129,7 +142,10 @@ void FractionBaseWidget::paintRatio(QPainter & paint, ratio tmp_ratio, int & x_p
 	return;
 }
 
-void FractionBaseWidget::paintMiddle(QPainter & paint, const QString& paint_str, int & x_pos, QFontMetrics & fm, QColor color, bool addMargin)
+void FractionBaseWidget::paintMiddle(QPainter & paint, 
+                                     const QString &paint_str, 
+                                     int & x_pos, int & y_pos, QFontMetrics & fm, 
+                                     const QColor &color, bool addMargin)
 {
 	// get the pen, font height and font width
 	QPen pen = paint.pen();
@@ -139,7 +155,7 @@ void FractionBaseWidget::paintMiddle(QPainter & paint, const QString& paint_str,
 	// paint the string
 	pen.setColor(color);
 	paint.setPen(pen);
-	paint.drawText(x_pos, fontHeight + 5 - fontHeight / 2, fontWidth, fontHeight, AlignCenter, paint_str);
+	paint.drawText(x_pos, y_pos + fontHeight - fontHeight / 2, fontWidth, fontHeight, Qt::AlignCenter, paint_str);
 
 	// move the x position to the right by adding the width used for
 	// painting the string and a margin
