@@ -4,6 +4,10 @@
     begin                : Tue Mar 08 17:20:00 CET 2002
     copyright            : (C) 2001 - 2004 by Sebastian Stein, Eva Brucherseifer
     email                : seb.kde@hpfsc.de
+
+    copyright            : (C) 2008 by Tadeu Araujo, tadeu.araujo@ltia.fc.unesp.br
+    copyright            : (C) 2008 by Danilo Balzaque, danilo.balzaque@ltia.fc.unesp.br
+    copyright            : (C) 2008 by Tiago Porangaba, tiago.porangaba@ltia.fc.unesp.br
  ***************************************************************************/
 
 /***************************************************************************
@@ -32,99 +36,102 @@
 #include <kdebug.h>
 
 #include "settingsclass.h"
+#include "statisticsbarwidget.h"
 
 /* constructor */
 StatisticsView::StatisticsView(QWidget * parent):
-		QWidget(parent), m_count(0), m_correct(0)
+		QFrame(parent), m_count(0), m_correct(0)
 {
 #ifdef DEBUG
 	kDebug() << "constructor StatisticsView()";
 #endif
-
 	// load statistics from config file
 	m_count = SettingsClass::count();
 	m_correct = SettingsClass::correct();
+	m_skipped = SettingsClass::skipped();
+
+	defaultFont = SettingsClass::taskFont();
+	defaultFont.setBold( true );
+	defaultFont.setPointSize(28);
 
 	QPalette pal;
 
-	/* set the caption of the window */
-	//	setCaption(i18n("Statistics"));
-
-	/* add a layout as a base */
-	layout1 = new QVBoxLayout(this);
-	layout1->setSpacing(6);
-	layout1->setMargin(6);
-
-	/* now add a v-spacer */
-	QSpacerItem * v_spacer = new QSpacerItem(1, 1);
-	layout1->addItem(v_spacer);
-
 	/* create a grid to show the labels */
 	labelGrid = new QGridLayout();
-        layout1->addItem( labelGrid );
+	setLayout(labelGrid);
 
-	/* add 6 labels to the grid */
-	info1Label = new QLabel(this);
-	info1Label->setText(i18n("Tasks so far:"));
-	labelGrid->addWidget(info1Label, 1, 0);
+	labelGrid->setColumnStretch(0,1);
+	labelGrid->setColumnStretch(6,1);
+
+	labelGrid->setColumnMinimumWidth(5,220);
+	labelGrid->setColumnMinimumWidth(2,30);
 
 	result1Label = new QLabel(this);
-	labelGrid->addWidget(result1Label, 1, 1);
+	labelGrid->addWidget(result1Label, 1, 1, 2, 1);
+	result1Label->setFont(defaultFont);
+	result1Label->setAlignment(Qt::AlignCenter);
 	result1Label->setToolTip(
 	              i18n("This is the current total number of solved tasks."));
 
+	defaultFont.setBold( false );
+	defaultFont.setPointSize(10);
+
+	info1Label = new QLabel(this);
+	info1Label->setText(i18n("Questions:"));
+	info1Label->setFont(defaultFont);
+	info1Label->setAlignment(Qt::AlignCenter);
+	labelGrid->addWidget(info1Label, 0, 1);
+  	
 	info2Label = new QLabel(this);
 	info2Label->setText(i18nc("@info:status the number of correct answers", "Correct:"));
-	labelGrid->addWidget(info2Label, 2, 0);
+	info2Label->setFont(defaultFont);
+	info2Label->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
+	labelGrid->addWidget(info2Label, 0, 3);	
 
-	result2Label = new QLabel(this);
-
-	/* set green text color for this label */
-	pal = result2Label->palette();
-    pal.setColor(QPalette::Active, QPalette::Foreground, QColor(6, 179, 0));
-    pal.setColor(QPalette::Inactive, QPalette::Foreground, QColor(6, 179, 0));
-	result2Label->setPalette(pal);
-
-	labelGrid->addWidget(result2Label, 2, 1);
-	result2Label->setToolTip(
-	              i18n("This is the current total number of correctly solved tasks."));
+	info4Label = new QLabel(this);
+	info4Label->setText(i18nc("@info:status the number of incorrect answers", "Incorrect:"));
+	info4Label->setFont(defaultFont);
+	info4Label->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
+	labelGrid->addWidget(info4Label, 1, 3);
 
 	info3Label = new QLabel(this);
-	info3Label->setText(i18nc("@info:status the number of incorrect answers", "Incorrect:"));
-	labelGrid->addWidget(info3Label, 3, 0);
+	info3Label->setText(i18nc("@info:status the number of skipped answers", "Skipped:"));
+	info3Label->setFont(defaultFont);
+	info3Label->setAlignment(Qt::AlignVCenter | Qt::AlignRight);
+	labelGrid->addWidget(info3Label, 2, 3);
 
-	result3Label = new QLabel(this);
+	defaultFont.setBold( true );
 
-	/* set red text color for this label */
-	pal = result3Label->palette();
-    pal.setColor(QPalette::Active, QPalette::Foreground, QColor(Qt::red));
-    pal.setColor(QPalette::Inactive, QPalette::Foreground, QColor(Qt::red));
-	result3Label->setPalette(pal);
-
-	labelGrid->addWidget(result3Label, 3, 1);
-	result3Label->setToolTip(
+	result2Label = new QLabel(this);
+	labelGrid->addWidget(result2Label, 0, 4);
+	result2Label->setFont(defaultFont);
+	result2Label->setToolTip(
+	              i18n("This is the current total number of correctly solved tasks."));
+	
+	result4Label = new QLabel(this);
+	labelGrid->addWidget(result4Label, 1, 4);
+	result4Label->setFont(defaultFont);
+	result4Label->setToolTip(
 	              i18n("This is the current total number of unsolved tasks."));
 
-	/* now add a v-spacer */
-	v_spacer = new QSpacerItem(1, 1);
-	layout1->addItem(v_spacer);
-
-	/* the Reset button */
-	buttonLayout = new QHBoxLayout();
-	layout1->addItem(buttonLayout);
-	resetBtn = new QPushButton(i18n("&Reset"), this);
-	QObject::connect(resetBtn, SIGNAL(clicked()), this, SLOT(resetStatistics()));
-	buttonLayout->addWidget(resetBtn);
-	resetBtn->setToolTip(i18n("Press the button to reset the statistics."));
-	QSpacerItem* spacer = new QSpacerItem(0,0);
-	buttonLayout->addItem(spacer);
-
-	/* calculate the statistics */
-	(void) calc();
+	result3Label = new QLabel(this);
+	labelGrid->addWidget(result3Label, 2, 4);
+	result3Label->setFont(defaultFont);
+	result3Label->setToolTip(
+	              i18n("This is the current total number of skipped tasks."));
 
 	// add tooltip and qwhatsthis help to the widget
 	setToolTip(i18n("This part of the window shows the statistics."));
 	setWhatsThis( i18n("This part of the window shows the statistics.  Each exercise you do is counted. You can reset the statistics by clicking on the button below. Also, if you do not want to see the statistics, use the vertical bar on the left to reduce the size of this window part."));
+
+	QBoxLayout * cLayout = new QBoxLayout(QBoxLayout::LeftToRight, this);
+
+	statisticsBar = new StatisticsBarWidget(this);
+	labelGrid->addWidget(statisticsBar, 0, 5, 3, 1);
+	labelGrid->addLayout(cLayout,0,5,Qt::AlignCenter);
+
+	/* calculate the statistics */
+	(void) calc();
 }
 
 /* destructor */
@@ -136,6 +143,7 @@ StatisticsView::~StatisticsView()
 	// save statistics for next run
 	SettingsClass::setCount(m_count);
 	SettingsClass::setCorrect(m_correct);
+	SettingsClass::setSkipped(m_skipped);
 	SettingsClass::self()->writeConfig();
 
 	/* no need to delete any child widgets, Qt does it by itself */
@@ -149,6 +157,14 @@ void StatisticsView::addCorrect()
 	(void) calc(); /* repaint the statistics */
 }
 
+/* called, if a task is skipped */
+void StatisticsView::addSkipped()
+{
+	++m_count;
+	++m_skipped;
+	(void) calc(); /* repaint the statistics */
+}
+
 /* called, if a task was solved wrong */
 void StatisticsView::addWrong()
 {
@@ -162,6 +178,8 @@ void StatisticsView::addWrong()
 /* recalculates the statistics and changes the corresponding labels */
 void StatisticsView::calc()
 {
+	statisticsBar->updateBar(m_correct, m_skipped, m_count);
+
 	QString new_text;
 	QString number;
 
@@ -171,16 +189,21 @@ void StatisticsView::calc()
 	/* we have to be careful with division by 0 */
 	if (m_count == 0)
 	{
-		result2Label->setText("- (- %)");
-		result3Label->setText("- (- %)");
+		result2Label->setText("0 (0 %)");
+		result3Label->setText("0 (0 %)");
+		result4Label->setText("0 (0 %)");
 	} else {
 		/* set the correct label */
-		new_text = QString("%1 (%2 %)").arg(m_correct).arg(int(double(m_correct) / m_count * 100));
+		new_text = QString("%1").arg(m_correct);
 		result2Label->setText(new_text);
 
-		/* set the incorrect label */
-		new_text = QString("%1 (%2 %)").arg(m_count - m_correct).arg(int(double(m_count - m_correct) / m_count * 100));
+		/* set the correct label */
+		new_text = QString("%1").arg(m_skipped);
 		result3Label->setText(new_text);
+
+		/* set the incorrect label */
+		new_text = QString("%1").arg(m_count - m_correct - m_skipped);
+		result4Label->setText(new_text);
 	}
 }
 
@@ -191,5 +214,6 @@ void StatisticsView::resetStatistics()
 {
 	m_count = 0;
 	m_correct = 0;
+	m_skipped = 0;
 	(void) calc();
 }
