@@ -22,7 +22,7 @@
 #include <kdialog.h>
 #include <unistd.h>
 
-#include "indi/libs/lilxml.h"
+#include <lilxml.h>
 #include "ui_devmanager.h"
 
 class QTreeWidgetItem;
@@ -97,7 +97,8 @@ public:
     INDIDriver(KStars *ks);
     ~INDIDriver();
 
-    bool readXMLDriver();
+    bool readXMLDrivers();
+    void processXMLDriver(QString & driverName);
 
     bool buildDriversList( XMLEle *root, char errmsg[]);
     bool buildDeviceGroup  (XMLEle *root, char errmsg[]);
@@ -114,7 +115,6 @@ public:
   
     void saveDevicesToDisk();
     int getINDIPort();
-    int activeDriverCount();
     bool isDeviceRunning(const QString &deviceLabel);
   
     void saveHosts();
