@@ -430,11 +430,11 @@ public:
     QRect scaledRect();
 
     /**
-        *@return whether the give QPoint is on the SkyMap.
+        *@return whether the given QPoint is on the SkyMap.
         */
     bool onScreen( QPoint &point );
     /**
-        *@return whether the give QPointF is on the SkyMap.
+        *@return whether the given QPointF is on the SkyMap.
         */
 
     bool onScreen( QPointF &pointF );
@@ -583,12 +583,12 @@ public slots:
     /**@short Popup menu function: Display 1st-Generation DSS image with the Image Viewer.
     	*@note the URL is generated using the coordinates of ClickedPoint.
     	*/
-    void slotDSS( void );
+    void slotDSS();
 
-    /**@short Popup menu function: Display 2nd-Generation DSS image with the Image Viewer.
+    /**@short Popup menu function: Display Sloan Digital Sky Survey image with the Image Viewer.
     	*@note the URL is generated using the coordinates of ClickedPoint.
     	*/
-    void slotDSS2( void );
+		void slotSDSS();
 
     /**@short Popup menu function: Show webpage about ClickedObject
     	*(only available for some objects). 
@@ -833,6 +833,53 @@ private:
     	*/
     bool unusablePoint ( const QPointF &p );
 
+    /** Zoom in a large amount or a small amount depending on the keyboard modifier
+     * @param modifier
+     */
+    void zoomIn( const int modifier );
+
+    /** Zoom in a large amount or a small amount depending on the keyboard modifier 
+     * @param modifier
+     */
+    void zoomOut( const int modifier );
+
+    /** Calculate the zoom factor for the given keyboard modifier
+     * @param modifier
+     */
+    double zoomFactor( const int modifier );
+
+    /** calculate the magnitude factor (1, .5, .2, or .1) for the given
+     * keyboard modifier.
+     * @param modifier
+     */
+    double magFactor( const int modifier );
+
+    /** Decrease the magnitude limit by a step size determined by the
+     * keyboard modifier.
+     * @param modifier
+     */
+    void decMagLimit( const int modifier );
+
+     /** Increase the magnitude limit by a step size determined by the
+     * keyboard modifier.
+     * @param modifier
+     */
+    void incMagLimit( const int modifier );
+
+    /** Convenience routine to either zoom in or incraase mag limit
+     * depending on the Alt modifier.  The Shift and Control modiifers
+     * will adjust the size of the zoom or the mag step.
+     * @param modifier 
+     */
+    void zoomInOrMagStep( const int modifier );
+
+    /** Convenience routine to either zoom out or decraase mag limit
+     * depending on the Alt modifier.  The Shift and Control modiifers
+     * will adjust the size of the zoom or the mag step.
+     * @param modifier 
+     */
+    void zoomOutOrMagStep( const int modifier );
+
     bool mouseButtonDown, midMouseButtonDown;
     bool mouseMoveCursor;  // true if mouseMoveEvent; needed by setMouseMoveCursor
     bool slewing, clockSlewing;
