@@ -10,26 +10,25 @@
  *   (at your option) any later version.                                   *
  ***************************************************************************/
 
-#include "config.h"
 #include "ktouchstatisticsdata.h"
 
-#include <qfile.h>
-#include <qstringlist.h>
-#include <qdom.h>
+// QT includes
+
+#include <QFile>
+#include <QTextStream>
+#include <QtXml/QDomNode>
+
+// KDE includes
 
 #include <kdebug.h>
 #include <klocale.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <kio/netaccess.h>
 
-#ifdef HAVE_SSTREAM
-#  include <sstream>
-#else
-#  include <strstream>
-#endif
+#include <sstream>
 
-// remove this define if XML output should be made uncompressed
-#define COMPRESSED_XML_STATISTICS
+// remove this define if XML output should not be compressed (for debug purposes only)
+// #define COMPRESSED_XML_STATISTICS  << dosn't work for me
 
 
 // *** KTouchCharStats ***
@@ -59,7 +58,7 @@ void KTouchLevelStats::clear() {
 }
 // ----------------------------------------------------------------------------
 
-bool KTouchLevelStats::read(QDomNode in) {
+bool KTouchLevelStats::read(const QDomNode &in) {
 	// read the level number
 	QDomNode n = in.attributes().namedItem("Number");
 	bool ok;
@@ -98,23 +97,20 @@ bool KTouchLevelStats::read(QDomNode in) {
 	n = in.namedItem("Time");
 	if (!n.isNull()) {
 		QString timestring = n.firstChild().nodeValue();
-		if (timestring != QString::null) 
+		if (!timestring.isEmpty()) 
 			m_timeRecorded = QDateTime::fromString(timestring, Qt::ISODate);
 	}
 	// read characters
 	n = in.namedItem("CharStats");
 	if (!n.isNull()) {
 		QString char_data = n.firstChild().nodeValue();
-		//kdDebug() << "'" << char_data << "'" << endl;
-#ifdef HAVE_SSTREAM
-		std::stringstream strm(std::string(char_data.local8Bit()));
-#else
-		std::istrstream strm(char_data.local8Bit(), char_data.local8Bit().length());
-#endif
+		//kDebug() << "'" << char_data << "'";
+		// FIXME : get rid of stringstream here
+		std::stringstream strm(std::string(char_data.toLocal8Bit()));
 		int ch, correct, wrong;
 		while (strm >> ch >> correct >> wrong)
 			m_charStats.insert( KTouchCharStats(QChar(ch), correct, wrong) );
-		//kdDebug() << m_charStats.size() << endl;
+		//kDebug() << m_charStats.size();
 	}
 	return true;
 }
@@ -136,7 +132,7 @@ void KTouchLevelStats::write(QDomDocument& doc, QDomElement& root) const {
 	// add char stats
 	QString char_data;
     // we append for each missed char the char-unicode and the counters
-	for (std::set<KTouchCharStats>::const_iterator it=m_charStats.begin(); it!=m_charStats.end(); ++it)
+	for (QSet<KTouchCharStats>::const_iterator it=m_charStats.begin(); it!=m_charStats.end(); ++it)
 		char_data += QString(" %1 %2 %3").arg(it->m_char.unicode())
 										 .arg(it->m_correctCount).arg(it->m_wrongCount);
 	e = doc.createElement("CharStats");
@@ -151,7 +147,7 @@ void KTouchLevelStats::addCorrectChar(QChar key) {
 	++m_totalChars;
     // we only count non-space characters
 	if (key!=QChar(' ')) {
-		std::set<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
+		QSet<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
 		if (it==m_charStats.end())
 			m_charStats.insert( KTouchCharStats(key,1,0) );
 		else
@@ -173,7 +169,7 @@ void KTouchLevelStats::addWrongChar(QChar key) {
 	++m_totalChars;
 	if (key==QChar(8) || key==QChar(' ')) 
 		return; // don't remember wrong backspaces or spaces
-	std::set<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
+	QSet<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
 	if (it==m_charStats.end())
 		m_charStats.insert( KTouchCharStats(key,0,1) );
 	else
@@ -214,7 +210,7 @@ void KTouchSessionStats::clear() {
 }
 // ----------------------------------------------------------------------------
 
-bool KTouchSessionStats::read(QDomNode in) {
+bool KTouchSessionStats::read(const QDomNode &in) {
 	// read time
 	QDomNode n = in.attributes().namedItem("Time");
 	bool ok;
@@ -244,18 +240,15 @@ bool KTouchSessionStats::read(QDomNode in) {
 	n = in.namedItem("Time");
 	if (!n.isNull()) {
 		QString timestring = n.firstChild().nodeValue();
-		if (timestring != QString::null) 
+		if (!timestring.isEmpty()) 
 			m_timeRecorded = QDateTime::fromString(timestring, Qt::ISODate);
 	}
 	// read level numbers
 	n = in.namedItem("LevelNums");
 	if (!n.isNull()) {
 		QString str = n.firstChild().nodeValue();
-#ifdef HAVE_SSTREAM
-		std::stringstream strm(std::string(str.local8Bit()));
-#else
-		std::istrstream strm(str.local8Bit(), str.local8Bit().length());
-#endif
+		// FIXME: get rid of stringstream here
+		std::stringstream strm(std::string(str.toLocal8Bit()));
 		int l;
 		while (strm >> l)
 			m_levelNums.insert(l);
@@ -264,16 +257,13 @@ bool KTouchSessionStats::read(QDomNode in) {
 	n = in.namedItem("CharStats");
 	if (!n.isNull()) {
 		QString char_data = n.firstChild().nodeValue();
-		//kdDebug() << "'" << char_data << "'" << endl;
-#ifdef HAVE_SSTREAM
-		std::stringstream strm(std::string(char_data.local8Bit()));
-#else
-		std::istrstream strm(char_data.local8Bit(), char_data.local8Bit().length());
-#endif
+		//kDebug() << "'" << char_data << "'";
+		// FIXME: get rid of stringstream here
+		std::stringstream strm(std::string(char_data.toLocal8Bit()));
 		int ch, correct, wrong;
 		while (strm >> ch >> correct >> wrong)
 			m_charStats.insert( KTouchCharStats(QChar(ch), correct, wrong) );
-		//kdDebug() << m_charStats.size() << endl;
+		//kDebug() << m_charStats.size();
 	}
 
 	return true;
@@ -294,7 +284,7 @@ void KTouchSessionStats::write(QDomDocument& doc, QDomElement& root) const {
 	session.appendChild(e);
 	// add levels
 	QString level_nums;
-	for (std::set<unsigned int>::const_iterator it = m_levelNums.begin(); it != m_levelNums.end(); ++it)
+	for (QSet<unsigned int>::const_iterator it = m_levelNums.begin(); it != m_levelNums.end(); ++it)
 		level_nums += QString( " %1").arg(*it);
 	e = doc.createElement("LevelNums");
 	tn = doc.createTextNode(level_nums);
@@ -303,7 +293,7 @@ void KTouchSessionStats::write(QDomDocument& doc, QDomElement& root) const {
 	// add char data
 	QString char_data;
     // we append for each missed char the char-unicode and the counters
-	for (std::set<KTouchCharStats>::const_iterator it=m_charStats.begin(); it!=m_charStats.end(); ++it)
+	for (QSet<KTouchCharStats>::const_iterator it=m_charStats.begin(); it!=m_charStats.end(); ++it)
 		char_data += QString(" %1 %2 %3").arg(it->m_char.unicode())
 				.arg(it->m_correctCount).arg(it->m_wrongCount);
 	e = doc.createElement("CharStats");
@@ -318,7 +308,7 @@ void KTouchSessionStats::addCorrectChar(QChar key) {
 	++m_totalChars;
     // we only count non-space characters
 	if (key!=QChar(' ')) {
-		std::set<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
+		QSet<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
 		if (it==m_charStats.end())
 			m_charStats.insert( KTouchCharStats(key,1,0) );
 		else
@@ -340,7 +330,7 @@ void KTouchSessionStats::addWrongChar(QChar key) {
 	++m_totalChars;
 	if (key==QChar(8) || key==QChar(' ')) 
 		return; // don't remember wrong backspaces or spaces
-	std::set<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
+	QSet<KTouchCharStats>::iterator it = m_charStats.find( KTouchCharStats(key,0,0) );
 	if (it==m_charStats.end())
 		m_charStats.insert( KTouchCharStats(key,0,1) );
 	else
@@ -357,14 +347,14 @@ void KTouchSessionStats::addWrongChar(QChar key) {
 // *** KTouchLectureStats ***
 
 void KTouchLectureStats::clear() {
-	m_lectureTitle = QString::null;
-	m_lectureURL = QString::null;
+	m_lectureTitle.clear();
+	m_lectureURL = "";
 	m_levelStats.clear();
 	m_sessionStats.clear();
 }
 // ----------------------------------------------------------------------------
 
-bool KTouchLectureStats::read(QDomNode in) {
+bool KTouchLectureStats::read(const QDomNode &in) {
 	clear();
 	// read the URL
 	QDomNode n = in.namedItem("URL");
@@ -376,10 +366,10 @@ bool KTouchLectureStats::read(QDomNode in) {
 	// read the level stats
 	n = in.namedItem("AllLevelStats");
 	if (!n.isNull()) {
-		// loop over all childs and read levels
+		// loop over all children and read levels
 		QDomNode levelNode = n.firstChild();
 		while (!levelNode.isNull()) {
-			//kdDebug() << "[KTouchLectureStats::read]  Reading level stats" << endl;
+			//kDebug() << "[KTouchLectureStats::read]  Reading level stats";
 			KTouchLevelStats tmp;
 			if (tmp.read(levelNode))  m_levelStats.push_back(tmp);
 			levelNode = levelNode.nextSibling();
@@ -388,10 +378,10 @@ bool KTouchLectureStats::read(QDomNode in) {
 	// read the session stats
 	n = in.namedItem("AllSessionStats");
 	if (!n.isNull()) {
-		// loop over all childs and read sessions
+		// loop over all children and read sessions
 		QDomNode sessionNode = n.firstChild();
 		while (!sessionNode.isNull()) {
-			//kdDebug() << "[KTouchLectureStats::read]  Reading session stats" << endl;
+			//kDebug() << "[KTouchLectureStats::read]  Reading session stats";
 			KTouchSessionStats tmp;
 			if (tmp.read(sessionNode))  m_sessionStats.push_back(tmp);
 			sessionNode = sessionNode.nextSibling();
@@ -421,7 +411,7 @@ void KTouchLectureStats::write(QDomDocument& doc, QDomElement& root) const {
 	// store level stats
 	QDomElement levelStatsElement = doc.createElement("AllLevelStats");
 	lecture.appendChild(levelStatsElement);
-	for (QValueVector<KTouchLevelStats>::ConstIterator it = m_levelStats.begin(); 
+	for (QList<KTouchLevelStats>::ConstIterator it = m_levelStats.begin(); 
 		it != m_levelStats.end(); ++it)
 	{
 		it->write(doc, levelStatsElement);
@@ -429,7 +419,7 @@ void KTouchLectureStats::write(QDomDocument& doc, QDomElement& root) const {
 	// store session stats
 	QDomElement sessionStatsElement = doc.createElement("AllSessionStats");
 	lecture.appendChild(sessionStatsElement);
-	for (QValueVector<KTouchSessionStats>::ConstIterator it = m_sessionStats.begin(); 
+	for (QList<KTouchSessionStats>::ConstIterator it = m_sessionStats.begin(); 
 			it != m_sessionStats.end(); ++it)
 	{
 		it->write(doc, sessionStatsElement);
@@ -442,12 +432,18 @@ void KTouchLectureStats::write(QDomDocument& doc, QDomElement& root) const {
 // *** KTouchStatisticsData ***
 
 void KTouchStatisticsData::clear() {
+	m_userName = i18n("Default User");
 	m_lectureStats.clear();
 }
 // ----------------------------------------------------------------------------
 
-bool KTouchStatisticsData::read(QWidget * window, const KURL& url) {
+// Reads all user statistics into the map 'stats'.
+bool KTouchStatisticsData::readUserStats(QWidget * window, const KUrl& url, 
+	QMap<QString, KTouchStatisticsData>& stats)
+{
+//	kdDebug() << "[KTouchStatisticsData::readUserStats]" << endl;
 	if (url.isEmpty()) return false;
+
     // Ok, first download the contents as usual using the KIO lib
     // File is only downloaded if not local, otherwise it's just opened
     QString target;
@@ -455,8 +451,10 @@ bool KTouchStatisticsData::read(QWidget * window, const KURL& url) {
     if (KIO::NetAccess::download(url, target, window)) {
         // Ok, that was successful, store the lectureURL and read the file
         QFile infile(target);
-        if ( !infile.open( IO_ReadOnly ) )
+        if ( !infile.open( QIODevice::ReadOnly ) ) {
+			// FIXME : what about removing the temp file?
             return false;   // Bugger it... couldn't open it...
+		}
 		QDomDocument doc;
 		
 #ifdef COMPRESSED_XML_STATISTICS
@@ -467,31 +465,64 @@ bool KTouchStatisticsData::read(QWidget * window, const KURL& url) {
 #else
 		doc.setContent( &infile );
 #endif // COMPRESSED_XML_STATISTICS
-		result = readStats(doc);
+
+//		kdDebug() << "[KTouchStatisticsData::readUserStats]  reading user stats..." << endl;
+
+		// here we read all available statistics store in the file
+		QDomNode n = doc.namedItem("KTouchStatisticsData");
+		if (!n.isNull()) {
+//			kdDebug() << "Entries = " << doc.elementsByTagName("KTouchStatistics").count() << endl;
+			n = n.namedItem("KTouchStatistics");
+		}
+		else
+			n = doc.namedItem("KTouchStatistics"); // read in old statistics
+		// loop over all entries
+		result = true;
+		while (result && !n.isNull()) {
+			KTouchStatisticsData userstats;
+			result = userstats.readStats(n);
+			// only add if reading stat data worked
+//			kdDebug() << "result = " << result << endl;
+			if (result) {
+				stats[userstats.m_userName] = userstats;
+			}
+			n = n.nextSibling();
+		}
 	}
     KIO::NetAccess::removeTempFile(target);
     return result;
 }
 // ----------------------------------------------------------------------------
 
-bool KTouchStatisticsData::write(QWidget * window, const KURL& url) const {
+// Writes all user statistics in the map 'stats'.
+bool KTouchStatisticsData::writeUserStats(QWidget * window, const KUrl& url, 
+	const QMap<QString, KTouchStatisticsData>& stats) 
+{
 	// create the XML document
 	QDomDocument doc;
-	writeStats(doc);
+	// add root node
+	QDomElement root = doc.createElement( "KTouchStatisticsData" );
+	doc.appendChild(root);
+
+	for (QMap<QString, KTouchStatisticsData>::const_iterator it = stats.constBegin();
+		it != stats.constEnd(); ++it) 
+	{
+		(*it).writeStats(doc, root);
+	}
 
 	// and save it
     QString tmpFile;
-    KTempFile *temp=0;
+    KTemporaryFile *temp=0;
     if (url.isLocalFile())
         tmpFile=url.path();         // for local files the path is sufficient
     else {
-        temp=new KTempFile;         // for remote files create a temporary file first
-        temp->setAutoDelete(true);  // so we don't have to delete the file ourselves
-        tmpFile=temp->name();
+        temp=new KTemporaryFile;         // for remote files create a temporary file first
+        temp->open();
+        tmpFile=temp->fileName();
     }
 
     QFile outfile(tmpFile);
-    if ( !outfile.open( IO_WriteOnly ) ) {
+    if ( !outfile.open( QIODevice::WriteOnly ) ) {
         if (temp)  delete temp;
         // kdDebug() << "Error creating lecture file!" << endl;
         return false;
@@ -499,10 +530,10 @@ bool KTouchStatisticsData::write(QWidget * window, const KURL& url) const {
 	
 #ifdef COMPRESSED_XML_STATISTICS
 	QByteArray array;
-	QTextStream out(array, IO_WriteOnly);
+	QTextStream out(array, QIODevice::WriteOnly);
 	out << doc.toString();
 	array = qCompress(array);
-	outfile.writeBlock(array);
+	outfile.write(array);
 #else
 	QTextStream out( &outfile );
     out << doc.toString();
@@ -518,34 +549,54 @@ bool KTouchStatisticsData::write(QWidget * window, const KURL& url) const {
 }
 // ----------------------------------------------------------------------------
 
-bool KTouchStatisticsData::readStats(QDomDocument& doc) {
+bool KTouchStatisticsData::readStats(const QDomNode &in) {
 	clear(); // clear current data
+
+	QDomNode n = in.namedItem("User");
+	if (!n.isNull()) 	m_userName = n.firstChild().nodeValue();
+	else 				m_userName = i18n("Default User");
+	if (m_userName.isEmpty())	m_userName = i18n("Default User");
+	if (m_userName.isEmpty())	m_userName = "Default User";
+
+//	kDebug() << "[KTouchStatisticsData::readStats]  reading user stats...";
+	kDebug() << "[KTouchStatisticsData::readStats]  User = '" << m_userName << "'";
+
 	// retrieve the KTouchStatistics statistics
-	QDomNodeList entries = doc.elementsByTagName("LectureStats");
+	n = in.namedItem("LectureStats");
 	bool result = true;
-	unsigned int i=0;
-	while (result && i < entries.count()) {
-		KTouchLectureStats stats;
-		result = stats.read(entries.item(i));
+	while (result && !n.isNull()) {
+		KTouchLectureStats tmp;
+		result = tmp.read(n);
 		// do we have a valid URL?
-		if (stats.m_lectureURL.isEmpty())  result = false;
+		if (tmp.m_lectureURL.isEmpty())  result = false;
 		if (result)
-			m_lectureStats[stats.m_lectureURL] = stats;
-		++i;
+			m_lectureStats[tmp.m_lectureURL] = tmp;
+		n = n.nextSibling();
 	}
+
 	return result;
 }
 // ----------------------------------------------------------------------------
 
-void KTouchStatisticsData::writeStats(QDomDocument& doc) const {
-	QDomElement root = doc.createElement( "KTouchStatistics" );
-	doc.appendChild(root);
-	// Store Lecture statistics
+void KTouchStatisticsData::writeStats(QDomDocument& doc, QDomElement& root) const {
+	kDebug() << "[KTouchStatisticsData::writeStats]  user = '" << m_userName << "'";
+
+	QDomElement user_data = doc.createElement( "KTouchStatistics" );
+	root.appendChild(user_data);
+
+	// store user name
+	QString username = m_userName;
+	if (username.isEmpty()) 	username = i18n("Default User");
+	QDomElement userElement = doc.createElement("User");
+	QDomText userText = doc.createTextNode(username);
+	userElement.appendChild(userText);
+	user_data.appendChild(userElement);
+
+	// store lecture statistics
 	LectureStatsMap::ConstIterator it = m_lectureStats.begin();
 	while (it != m_lectureStats.end()) {
-		it.data().write(doc, root);
+		it.value().write(doc, user_data);
 		++it;
 	}
 }
 // ----------------------------------------------------------------------------
-
