@@ -24,6 +24,7 @@
 #include "keduvocdocument.h"
 #include "keduvocexpression.h"
 #include "keduvoclesson.h"
+#include "keduvocleitnerbox.h"
 #include "keduvocwordtype.h"
 #include "kvtml2defs.h"
 
@@ -55,7 +56,7 @@ bool KEduVocKvtml2Writer::createXmlDocument( KEduVocDocument *doc, const QString
 {
     m_doc = doc;
 
-    m_domDoc = QDomDocument( "kvtml PUBLIC \"kvtml2.dtd\" \"http://edu.kde.org/kanagram/kvtml2.dtd\"" );
+    m_domDoc = QDomDocument( "kvtml PUBLIC \"kvtml2.dtd\" \"http://edu.kde.org/kvtml/kvtml2.dtd\"" );
     m_domDoc.appendChild( m_domDoc.createProcessingInstruction( "xml", "version=\"1.0\" encoding=\"UTF-8\"" ) );
     QDomElement domElementKvtml = m_domDoc.createElement( "kvtml" );
     m_domDoc.appendChild( domElementKvtml );
@@ -72,13 +73,6 @@ bool KEduVocKvtml2Writer::createXmlDocument( KEduVocDocument *doc, const QString
     writeIdentifiers( currentElement );
     domElementKvtml.appendChild( currentElement );
 
-    // tenses
-    currentElement = m_domDoc.createElement( KVTML_TENSES );
-    writeTenses( currentElement );
-    if ( currentElement.hasChildNodes() ) {
-        domElementKvtml.appendChild( currentElement );
-    }
-
     // entries
     currentElement = m_domDoc.createElement( KVTML_ENTRIES );
     if ( !writeEntries( currentElement ) ) {
@@ -101,6 +95,13 @@ bool KEduVocKvtml2Writer::createXmlDocument( KEduVocDocument *doc, const QString
         domElementKvtml.appendChild( currentElement );
     }
 
+    // leitner boxes
+    currentElement = m_domDoc.createElement( KVTML_LEITNERBOXES );
+    writeLeitnerBoxes( currentElement, m_doc->leitnerContainer() );
+    if ( currentElement.hasChildNodes() ) {
+        domElementKvtml.appendChild( currentElement );
+    }
+
     writeSynonymAntonymFalseFriend(domElementKvtml);
 
     m_domDoc.appendChild( domElementKvtml );
@@ -141,6 +142,9 @@ bool KEduVocKvtml2Writer::writeInformation( QDomElement &informationElement, con
         informationElement.appendChild( newTextElement( KVTML_COMMENT, m_doc->documentComment() ) );
     }
 
+    QDate today = QDate::currentDate();
+    informationElement.appendChild( newTextElement( KVTML_DATE, today.toString(QLatin1String("yyyy-MM-dd")) ) );
+
     // category
     if ( !m_doc->category().isEmpty() ) {
         informationElement.appendChild( newTextElement( KVTML_CATEGORY, m_doc->category() ) );
@@ -179,6 +183,12 @@ bool KEduVocKvtml2Writer::writeIdentifiers( QDomElement &identifiersElement )
             identifier.appendChild( personalpronouns );
         }
 
+        // tenses
+        foreach(const QString &tense, m_doc->identifier(i).tenseList() ) {
+            if ( !( tense.isNull() ) ) {
+                identifier.appendChild( newTextElement( KVTML_TENSE, tense ) );
+            }
+        }
         // add this identifier to the group
         identifiersElement.appendChild( identifier );
     }
@@ -326,16 +336,28 @@ bool KEduVocKvtml2Writer::writeRelated(QDomElement & parentElement, QList< KEduV
 bool KEduVocKvtml2Writer::writeArticle( QDomElement &articleElement, int language )
 {
     ///@todo only write if not empty
-    for (int num = KEduVocArticle::Singular; num <= KEduVocArticle::Plural; num++)
+    QMap<int, KEduVocWordFlag::Flags> numbers;
+    numbers[0] = KEduVocWordFlag::Singular;
+    numbers[1] = KEduVocWordFlag::Dual;
+    numbers[2] = KEduVocWordFlag::Plural;
+    QMap<int, KEduVocWordFlag::Flags> genders;
+    genders[0] = KEduVocWordFlag::Masculine;
+    genders[1] = KEduVocWordFlag::Feminine;
+    genders[2] = KEduVocWordFlag::Neuter;
+    QMap<int, KEduVocWordFlag::Flags> defs;
+    defs[0] = KEduVocWordFlag::Definite;
+    defs[1] = KEduVocWordFlag::Indefinite;
+
+    for (int num = 0; num <= 2; num++)
     {
         QDomElement numberElement = m_domDoc.createElement( KVTML_GRAMMATICAL_NUMBER[num] );
 
-        for (int def = KEduVocArticle::Definite; def <= KEduVocArticle::Indefinite; def++) {
+        for (int def = 0; def <= 1; def++) {
             QDomElement defElement = m_domDoc.createElement( KVTML_GRAMMATICAL_DEFINITENESS[def] );
 
-            for (int gen = KEduVocArticle::Masculine; gen <= KEduVocArticle::Neutral; gen++)
+            for (int gen = 0; gen <= 2; gen++)
             {
-                QString articleString = m_doc->identifier(language).article().article( KEduVocArticle::ArticleNumber(num), KEduVocArticle::ArticleDefiniteness(def), KEduVocArticle::ArticleGender(gen) );
+                QString articleString = m_doc->identifier(language).article().article(numbers[num] | genders[gen] | defs[def]);
                 if ( !articleString.isEmpty() ) {
                     defElement.appendChild( newTextElement( KVTML_GRAMMATICAL_GENDER[gen], articleString ) );
                 }
@@ -360,32 +382,27 @@ bool KEduVocKvtml2Writer::writeWordTypes( QDomElement &typesElement, KEduVocWord
         QDomElement typeDefinitionElement = m_domDoc.createElement( KVTML_CONTAINER );
         typeDefinitionElement.appendChild( newTextElement( KVTML_NAME, wordType->name() ) );
 
-        switch (wordType->wordType()) {
-        case KEduVocWordType::Noun:
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN ) );
-            break;
-        case KEduVocWordType::NounMale:
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN_MALE ) );
-            break;
-        case KEduVocWordType::NounFemale:
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN_FEMALE ) );
-            break;
-        case KEduVocWordType::NounNeutral:
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN_NEUTRAL ) );
-            break;
-        case KEduVocWordType::Verb:
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_VERB ) );
-            break;
-        case KEduVocWordType::Adjective:
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_ADJECTIVE ) );
-            break;
-        case KEduVocWordType::Adverb:
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_ADVERB ) );
-            break;
-        default:
-            // no special type, no tag
-            break;
+        if (wordType->wordType().testFlag(KEduVocWordFlag::Noun))
+        {
+            if (wordType->wordType().testFlag(KEduVocWordFlag::Masculine))
+                typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN_MALE ) );
+
+            else if (wordType->wordType().testFlag(KEduVocWordFlag::Feminine))
+                typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN_FEMALE ) );
+
+            else if (wordType->wordType().testFlag(KEduVocWordFlag::Neuter))
+                typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN_NEUTRAL ) );
+            else
+                typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_NOUN ) );
         }
+        else if (wordType->wordType().testFlag(KEduVocWordFlag::Verb))
+            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_VERB ) );
+
+        else if (wordType->wordType().testFlag(KEduVocWordFlag::Adjective))
+            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_ADJECTIVE ) );
+
+        else if (wordType->wordType().testFlag(KEduVocWordFlag::Adverb))
+            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, KVTML_SPECIALWORDTYPE_ADVERB ) );
 
 
 // child entries
@@ -413,15 +430,32 @@ bool KEduVocKvtml2Writer::writeWordTypes( QDomElement &typesElement, KEduVocWord
     return true;
 }
 
-
-bool KEduVocKvtml2Writer::writeTenses( QDomElement &tensesElement )
+bool KEduVocKvtml2Writer::writeLeitnerBoxes( QDomElement &leitnerParentElement, KEduVocLeitnerBox* parentContainer )
 {
-    foreach( const QString &tense, m_doc->tenseDescriptions() ) {
-        if ( !( tense.isNull() ) ) {
-            tensesElement.appendChild( newTextElement( KVTML_TENSE, tense ) );
-        }
-    }
+    foreach( KEduVocContainer* container, parentContainer->childContainers() ) {
+        KEduVocLeitnerBox* leitnerBox = static_cast<KEduVocLeitnerBox*>(container);
 
+        QDomElement containerElement = m_domDoc.createElement( KVTML_CONTAINER );
+        containerElement.appendChild( newTextElement( KVTML_NAME, leitnerBox->name() ) );
+
+        // child entries
+        foreach(KEduVocExpression *entry, leitnerBox->entries()) {
+            QDomElement entryElement = m_domDoc.createElement( KVTML_ENTRY );
+            entryElement.setAttribute( KVTML_ID, QString::number(m_allEntries.indexOf(entry)) );
+            for(int translation = 0; translation<m_doc->identifierCount(); translation++) {
+                if (entry->translation(translation)->leitnerBox()== leitnerBox) {
+                    QDomElement translationElement = m_domDoc.createElement( KVTML_TRANSLATION );
+                    // create <translation id="123">
+                    translationElement.setAttribute( KVTML_ID, QString::number(translation) );
+                    // append both
+                    entryElement.appendChild(translationElement);
+                }
+            }
+            containerElement.appendChild( entryElement );
+        }
+
+        leitnerParentElement.appendChild( containerElement );
+    }
     return true;
 }
 
@@ -593,11 +627,25 @@ bool KEduVocKvtml2Writer::writePersonalPronoun(QDomElement & pronounElement, con
         pronounElement.appendChild( m_domDoc.createElement( KVTML_DUAL_EXISTS ) );
     }
 
+
+    QMap<int, KEduVocWordFlag::Flags> numbers;
+    numbers[0] = KEduVocWordFlag::Singular;
+    numbers[1] = KEduVocWordFlag::Dual;
+    numbers[2] = KEduVocWordFlag::Plural;
+    QMap<int, KEduVocWordFlag::Flags> persons;
+    persons[0] = KEduVocWordFlag::First;
+    persons[1] = KEduVocWordFlag::Second;
+    persons[2] = (KEduVocWordFlag::Flags)((int)KEduVocWordFlag::Third | (int)KEduVocWordFlag::Masculine);
+    persons[3] = (KEduVocWordFlag::Flags)((int)KEduVocWordFlag::Third | (int)KEduVocWordFlag::Feminine);
+    persons[4] = (KEduVocWordFlag::Flags)((int)KEduVocWordFlag::Third | (int)KEduVocWordFlag::Neuter);
+
+
+
     // the actual pronouns
-    for ( int num = KEduVocConjugation::Singular; num <= KEduVocConjugation::Plural; num++ ) {
+    for ( int num = 0; num < 3; num++ ) {
         QDomElement numberElement = m_domDoc.createElement( KVTML_GRAMMATICAL_NUMBER[num] );
-        for ( int person = KEduVocConjugation::First; person <= KEduVocConjugation::ThirdNeutralCommon; person++ ) {
-            QString pronounString = pronoun.personalPronoun(KEduVocConjugation::ConjugationPerson(person), KEduVocConjugation::ConjugationNumber(num));
+        for ( int person = 0; person < 5; person++ ) {
+            QString pronounString = pronoun.personalPronoun(numbers[num] | persons[person]);
             if (!pronounString.isEmpty()) {
                 numberElement.appendChild( newTextElement( KVTML_GRAMMATICAL_PERSON[person], pronounString ));
             }
