@@ -24,14 +24,17 @@
 #include "MarbleControlBox.h"
 
 #include <QtCore/QAbstractItemModel>
+#include <QtCore/QBuffer>
+#include <QtCore/QByteArray>
+#include <QtCore/QDataStream>
 #include <QtCore/QtAlgorithms>
 #include <QtCore/QDebug>
+#include <QtCore/QTime>
 #include <QtCore/QTimer>
 #include <QtGui/QSortFilterProxyModel>
 #include <QtGui/QStandardItemModel>
 #include <QtGui/QTextFrame>
-
-#include "ui_MarbleControlBox.h"
+#include <QtNetwork/QHttp>
 
 #include <global.h>
 
@@ -42,12 +45,26 @@
 #include <MarbleDirs.h>
 #include <FileViewModel.h>
 #include "gps/GpxFileModel.h"
-#include "CustomSortFilterProxyModel.h"
+#include "PlaceMarkContainer.h"
+#include "MarblePlacemarkModel.h"
+#include "MarbleRunnerManager.h"
+#include "MathHelper.h"
+#include "MapThemeSortFilterProxyModel.h"
+
+#include "GeoOnfParser.h"
+#include "GeoDataDocument.h"
+
+using namespace Marble;
+
+#include "ui_MarbleControlBox.h"
+
+namespace Marble
+{
 
 class MarbleControlBoxPrivate
 {
  public:
-
+    MarbleControlBoxPrivate();
     MarbleWidget  *m_widget;
     QString        m_searchTerm;
     bool           m_searchTriggered;
@@ -57,13 +74,19 @@ class MarbleControlBoxPrivate
     QWidget              *m_legendWidget;
     QWidget              *m_mapViewWidget;
     QWidget              *m_currentLocationWidget;
+    QWidget              *m_currentLocation2Widget;
     QWidget              *m_fileViewWidget;
 
     QStandardItemModel     *m_mapThemeModel;
     QSortFilterProxyModel  *m_sortproxy;
-    CustomSortFilterProxyModel *m_mapSortProxy;
+    MapThemeSortFilterProxyModel *m_mapSortProxy;
+    
+    MarbleRunnerManager  *m_runnerManager;
 };
 
+MarbleControlBoxPrivate::MarbleControlBoxPrivate()
+{
+}
 
 // ================================================================
 
@@ -73,7 +96,7 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
       d( new MarbleControlBoxPrivate )
 {
     d->m_widget = 0;
-    d->m_searchTerm = QString();
+    d->m_searchTerm.clear();
     d->m_searchTriggered = false;
 
     d->uiWidget.setupUi( this );
@@ -97,10 +120,13 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
     d->uiWidget.toolBox->setCurrentIndex( 4 );
     d->m_fileViewWidget = d->uiWidget.toolBox->currentWidget();
 
+    d->uiWidget.toolBox->setCurrentIndex( 5 );
+    d->m_currentLocation2Widget = d->uiWidget.toolBox->currentWidget();
+
     d->m_sortproxy = new QSortFilterProxyModel( d->uiWidget.locationListView );
     d->uiWidget.locationListView->setModel( d->m_sortproxy );
 
-    d->m_mapSortProxy = new CustomSortFilterProxyModel( this );
+    d->m_mapSortProxy = new MapThemeSortFilterProxyModel( this );
 
 //  d->m_currentLocationWidget->hide(); // Current location tab is hidden
                                     //by default
@@ -109,6 +135,7 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
 
     //default
     setCurrentLocationTabShown( false );
+    setCurrentLocation2TabShown( false );
     setFileViewTabShown( false );
 
     setupGpsOption();
@@ -134,8 +161,10 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
     connect( d->uiWidget.moveDownButton,  SIGNAL( clicked() ),
              this,                        SIGNAL (moveDown() ) );
 
-    connect(d->uiWidget.searchLineEdit,   SIGNAL( textChanged( const QString& ) ),
-            this,                         SLOT( searchLineChanged( const QString& ) ) );
+    connect( d->uiWidget.searchLineEdit,  SIGNAL( textChanged( const QString& ) ),
+             this,                        SLOT( searchLineChanged( const QString& ) ) );
+    connect( d->uiWidget.searchLineEdit,  SIGNAL( returnPressed() ),
+             this,                        SLOT( searchReturnPressed() ) );
 
     connect( d->uiWidget.locationListView, SIGNAL( centerOn( const QModelIndex& ) ),
              this,                         SLOT( mapCenterOnSignal( const QModelIndex& ) ) );
@@ -153,6 +182,11 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
              this,                      SLOT( adjustForStill() ) );
 
     d->uiWidget.projectionComboBox->setEnabled( true );
+    
+    d->m_runnerManager = new MarbleRunnerManager( this );
+    
+    connect( d->m_runnerManager, SIGNAL( modelChanged(  MarblePlacemarkModel* ) ),
+             this,               SLOT( setLocations( MarblePlacemarkModel* ) ) );    
 }
 
 MarbleControlBox::~MarbleControlBox()
@@ -165,7 +199,7 @@ void MarbleControlBox::setMapThemeModel( QStandardItemModel *mapThemeModel ) {
     d->m_mapSortProxy->setSourceModel( d->m_mapThemeModel );
     d->m_mapSortProxy->sort( 0 );
     d->uiWidget.marbleThemeSelectView->setModel( d->m_mapSortProxy );
-    connect( d->m_mapThemeModel,       SIGNAL( rowsInserted ( QModelIndex, int, int) ),
+    connect( d->m_mapThemeModel,       SIGNAL( rowsInserted( QModelIndex, int, int ) ),
              this,                     SLOT( updateMapThemeView() ) );
     updateMapThemeView();
 }
@@ -205,21 +239,21 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
 
     // Make us aware of all the PlaceMarks in the MarbleModel so that
     // we can search them.
-    setLocations( d->m_widget->placeMarkModel() );
+    setLocations( static_cast<MarblePlacemarkModel*>(d->m_widget->placeMarkModel()) );
     d->uiWidget.locationListView->setSelectionModel( d->m_widget->placeMarkSelectionModel() );
 
 #ifndef KML_GSOC
     //set up everything for the FileModel
-    d->uiWidget.m_fileView->setModel( widget->gpxFileModel() );
+    d->uiWidget.m_fileView->setModel( widget->fileViewModel() );
 
     connect( d->uiWidget.m_fileView->selectionModel(),
 	     SIGNAL( selectionChanged( QItemSelection, QItemSelection )),
 	     this,
 	     SLOT( enableFileViewActions() ) );
     connect( d->uiWidget.m_saveButton,  SIGNAL( clicked() ) ,
-             widget->gpxFileModel(),    SLOT( saveFile() ) );
+             widget->fileViewModel(),    SLOT( saveFile() ) );
     connect( d->uiWidget.m_closeButton, SIGNAL( clicked() ) ,
-             widget->gpxFileModel(),    SLOT( closeFile() ) );
+             widget->fileViewModel(),    SLOT( closeFile() ) );
 #else
     FileViewModel* model = widget->fileViewModel();
     d->uiWidget.m_fileView->setModel( model );
@@ -255,7 +289,6 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
 
     connect( d->m_widget, SIGNAL( themeChanged( QString ) ),
              this,        SLOT( selectTheme( QString ) ) );
-    selectTheme( d->m_widget->mapThemeId() );
 
     connect( d->m_widget, SIGNAL( projectionChanged( Projection ) ),
              this,        SLOT( selectProjection( Projection ) ) );
@@ -276,12 +309,12 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
     //connect CurrentLoctaion signals
     connect( this, SIGNAL( gpsInputDisabled( bool ) ),
              d->m_widget, SLOT( setShowGps( bool ) ) );
-    connect( this, SIGNAL( gpsPositionChanged( double, double ) ),
-             d->m_widget, SLOT( changeCurrentPosition( double, double ) ) );
-    connect( d->m_widget, SIGNAL( mouseClickGeoPosition( double, double,
-                                            GeoDataPoint::Unit ) ),
-             this, SLOT( receiveGpsCoordinates ( double, double,
-                                                 GeoDataPoint::Unit) ) );
+    connect( this, SIGNAL( gpsPositionChanged( qreal, qreal ) ),
+             d->m_widget, SLOT( changeCurrentPosition( qreal, qreal ) ) );
+    connect( d->m_widget, SIGNAL( mouseClickGeoPosition( qreal, qreal,
+                                            GeoDataCoordinates::Unit ) ),
+             this, SLOT( receiveGpsCoordinates ( qreal, qreal,
+                                                 GeoDataCoordinates::Unit) ) );
 
     connect( d->m_widget, SIGNAL( timeout() ),
              this,        SIGNAL( updateGps() ) );
@@ -313,11 +346,14 @@ void MarbleControlBox::setWidgetTabShown( QWidget * widget,
 }
 
 
-void MarbleControlBox::setLocations(QAbstractItemModel* locations)
+void MarbleControlBox::setLocations(MarblePlacemarkModel* locations)
 {
+    QTime t;
+    t.start();
     d->m_sortproxy->setSourceModel( locations );
-//    d->m_sortproxy->setSortLocaleAware( true );
+    d->m_sortproxy->setSortLocaleAware( true );
     d->m_sortproxy->sort( 0 );
+    qDebug("MarbleControlBox (sort): Time elapsed: %d ms", t.elapsed());
 }
 
 int MarbleControlBox::minimumZoom() const
@@ -347,8 +383,8 @@ void MarbleControlBox::disableGpsInput( bool in )
     d->uiWidget.m_latComboBox->setEnabled( !in );
     d->uiWidget.m_lonComboBox->setEnabled( !in );
 
-    double t_lat = d->uiWidget.m_latSpinBox->value();
-    double t_lon = d->uiWidget.m_lonSpinBox->value();
+    qreal t_lat = d->uiWidget.m_latSpinBox->value();
+    qreal t_lon = d->uiWidget.m_lonSpinBox->value();
 
     if( d->uiWidget.m_lonComboBox->currentIndex() == 1 ){
         t_lon *= -1;
@@ -362,18 +398,18 @@ void MarbleControlBox::disableGpsInput( bool in )
     emit gpsInputDisabled( in );
 }
 
-void MarbleControlBox::receiveGpsCoordinates( double x, double y,
-                                              GeoDataPoint::Unit unit)
+void MarbleControlBox::receiveGpsCoordinates( qreal x, qreal y,
+                                              GeoDataCoordinates::Unit unit)
 {
     if ( d->uiWidget.m_catchGps->isChecked() ) {
         switch(unit){
-        case GeoDataPoint::Degree:
+        case GeoDataCoordinates::Degree:
             d->uiWidget.m_lonSpinBox->setValue( y );
             d->uiWidget.m_latSpinBox->setValue( x );
             emit gpsPositionChanged( y, x );
             break;
-        case GeoDataPoint::Radian:
-            double t_lat=0,t_lon=0;
+        case GeoDataCoordinates::Radian:
+            qreal t_lat=0,t_lon=0;
             t_lat = y * -RAD2DEG;
             t_lon = x * +RAD2DEG;
 
@@ -449,6 +485,11 @@ void MarbleControlBox::setFileViewTabShown( bool show )
     setWidgetTabShown( d->m_fileViewWidget, 4, show, title );
 }
 
+void MarbleControlBox::setCurrentLocation2TabShown( bool show )
+{
+    QString  title = tr( "Current Location" );
+    setWidgetTabShown( d->m_currentLocation2Widget, 5, show, title );
+}
 
 
 
@@ -476,12 +517,26 @@ void MarbleControlBox::resizeEvent ( QResizeEvent * )
 void MarbleControlBox::searchLineChanged(const QString &search)
 {
     d->m_searchTerm = search;
+    // if search line is empty, restore original geonames
+    if ( d->m_searchTerm.isEmpty() )
+        setLocations( static_cast<MarblePlacemarkModel*>( d->m_widget->placeMarkModel() ) );
     if ( d->m_searchTriggered )
         return;
     d->m_searchTriggered = true;
     QTimer::singleShot( 0, this, SLOT( search() ) );
 }
 
+void MarbleControlBox::searchReturnPressed()
+{
+    // do nothing if search term empty
+    if ( d->m_searchTerm.isEmpty() ) {
+        return;
+    } else {
+        d->m_runnerManager->newText( d->m_searchTerm );
+        return;
+    }
+}
+
 
 void MarbleControlBox::search()
 {
@@ -494,37 +549,33 @@ void MarbleControlBox::search()
 
 void MarbleControlBox::selectTheme( const QString &theme )
 {
-    qDebug() << "Entered selectTheme";
-    if ( !d->m_mapSortProxy )
+    if ( !d->m_mapSortProxy || !d->m_widget )
         return;
-    for ( int row = 0; row < d->m_mapSortProxy->rowCount(); ++row ) {
-        QModelIndex itIndexName = d->m_mapSortProxy->index( row, 1, QModelIndex() );
-        QModelIndex itIndex     = d->m_mapSortProxy->index( row, 0, QModelIndex() );
-        qDebug() << "Select Theme: " << theme << " Stored: " << d->m_mapSortProxy->data( itIndexName ).toString();
-        // If  we have found the theme in the theme model,
-        //     and it is not the one that we already have,
-        // then
-        //     set the new one in the ui.
-        if ( theme == d->m_mapSortProxy->data( itIndexName ).toString()
-             && itIndexName != d->uiWidget.marbleThemeSelectView->currentIndex() ) {
-            // Mark the correct picture for the selected map theme and
-            // also make sure it's shown.
-            d->uiWidget.marbleThemeSelectView->setCurrentIndex( itIndex );
-            d->uiWidget.marbleThemeSelectView->scrollTo( itIndex );
 
-            // Dig out the min and max zoom and set the slider.
-            //
-            // Since this slot is called when a change of map theme is
-            // already finished, we can use the widget to get the
-            // values.
-            if ( ! d->m_widget )
-                break;
-            d->uiWidget.zoomSlider->setMinimum( d->m_widget->map()->minimumZoom() );
-            d->uiWidget.zoomSlider->setMaximum( d->m_widget->map()->maximumZoom() );
+    // Check if the new selected theme is different from the current one
+    QModelIndex currentIndex = d->uiWidget.marbleThemeSelectView->currentIndex();
+    QString indexTheme = d->m_mapSortProxy->data( d->m_mapSortProxy->index( 
+                         currentIndex.row(), 1, QModelIndex() ) ).toString();
 
-            updateButtons( d->uiWidget.zoomSlider->value() );
-            // Break out of the loop
-            break;
+
+    d->uiWidget.zoomSlider->setMaximum( d->m_widget->map()->maximumZoom() );
+    updateButtons( d->uiWidget.zoomSlider->value() );
+    if ( theme != indexTheme ) {
+        /* indexTheme would be empty if the chosen map has not been set yet. As 
+        this needs to be done after the mapThemeId has been set, check if that is 
+        not empty first. The behaviour differs between Linux and Windows: on
+        Windows the reading of the settings is not delayed, thus the mapThemeId
+        is available earlier than on Linux.
+        */
+        if( indexTheme.isEmpty() && !d->m_widget->mapThemeId().isEmpty() ) {
+            QList<QStandardItem*> items = d->m_mapThemeModel->findItems( theme, Qt::MatchExactly, 1 );
+            if( items.size() >= 1 ) {
+                QModelIndex iterIndex = items.at( 0 )->index();
+                QModelIndex iterIndexName = d->m_mapSortProxy->mapFromSource( iterIndex.sibling( iterIndex.row(), 0 ) );
+
+                d->uiWidget.marbleThemeSelectView->setCurrentIndex( iterIndexName );
+                d->uiWidget.marbleThemeSelectView->scrollTo( iterIndexName );
+            }
         }
     }
 }
@@ -571,4 +622,6 @@ void MarbleControlBox::adjustForStill()
     }
 }
 
+}
+
 #include "MarbleControlBox.moc"
