@@ -18,12 +18,16 @@
 #define MARBLETHEMESELECTVIEW_H
 
 
-#include <QtCore/QDebug>
-#include <QtGui/QStandardItemModel>
+#include <QtCore/QModelIndex>
 #include <QtGui/QListView>
 
 #include "marble_export.h"
 
+class QResizeEvent;
+class QString;
+
+namespace Marble
+{
 
 class MARBLE_EXPORT MarbleThemeSelectView : public QListView
 {
@@ -48,4 +52,6 @@ class MARBLE_EXPORT MarbleThemeSelectView : public QListView
     Private  * const d;
 };
 
+}
+
 #endif // KATLASTHEMESELECTVIEW_H
