@@ -18,7 +18,9 @@
 #define TEXTURECOLORIZER_H
 
 #include <QtCore/QString>
-// FIXME: forward decl for QString
+
+namespace Marble
+{
 
 class ViewParams;
 
@@ -57,5 +59,6 @@ class TextureColorizer
     static uint texturepalette[16][512];
 };
 
+}
 
 #endif
