@@ -21,6 +21,9 @@
 
 #include "geodata_export.h"
 
+namespace Marble
+{
+
 class GeoDataIconStylePrivate;
 
 class GEODATA_EXPORT GeoDataIconStyle : public GeoDataColorStyle
@@ -33,6 +36,8 @@ class GEODATA_EXPORT GeoDataIconStyle : public GeoDataColorStyle
 
     void setIcon( const QPixmap& icon );
     QPixmap icon() const;
+    
+    void setIconPath( const QString& filename );
 
     void setHotSpot( const QPointF& hotSpot, GeoDataHotSpot::Units xunits, GeoDataHotSpot::Units yunits );
     const QPointF& hotSpot() const;
@@ -51,4 +56,6 @@ class GEODATA_EXPORT GeoDataIconStyle : public GeoDataColorStyle
     GeoDataIconStylePrivate * const d;
 };
 
+}
+
 #endif // GEODATAICONSTYLE_H
