@@ -24,12 +24,18 @@
 
 #include "GeoTagHandler.h"
 
-class DgmlVectorTagHandler : public GeoTagHandler {
-public:
-    DgmlVectorTagHandler();
-    virtual ~DgmlVectorTagHandler();
+namespace Marble
+{
+namespace dgml
+{
 
+class DgmlVectorTagHandler : public GeoTagHandler
+{
+public:
     virtual GeoNode* parse(GeoParser&) const;
 };
 
+}
+}
+
 #endif // DgmlVectorTagHandler_h
