@@ -17,6 +17,8 @@
 class QXmlAttributes;
 class QXmlParseException;
 
+namespace Marble
+{
 
 /**
 A very simple Xml Parser that will be based on the Gml Scema ( Geographic Markup Language )
@@ -26,7 +28,7 @@ A very simple Xml Parser that will be based on the Gml Scema ( Geographic Markup
 class GmlSax : public QXmlDefaultHandler
 {
 public:
-    GmlSax( double *lon, double *lat );
+    GmlSax( qreal *lon, qreal *lat );
 
     ~GmlSax();
 
@@ -42,10 +44,12 @@ public:
     bool characters( const QString &str );
 
 private:
-    double *m_lat;
-    double *m_lon;
+    qreal *m_lat;
+    qreal *m_lon;
 
     QString currentText;
 };
 
+}
+
 #endif
