@@ -13,6 +13,9 @@
 
 #include "AbstractLayerContainer.h"
 
+namespace Marble
+{
+
 /**
  * @brief holds a collection of tracks
  */
@@ -38,4 +41,6 @@ class TrackContainer : public AbstractLayerContainer
                        const BoundingBox &box );
 };
 
+}
+
 #endif
